import pandas as pd


def drop_tz_from_dataframe(df:pd.DataFrame):
    date_columns = df.select_dtypes(include=['datetime64[ns, UTC]']).columns
    for date_column in date_columns:
        df[date_column] = df[date_column].dt.tz_localize(None)


from typing import Dict, Generic, TypeVar
from django.db.models import QuerySet,Count
import types

T=TypeVar('T')
class SortingMixin(Generic[T]):
    """Overrides the `filter_queryset` method"""

    sorting_aliases: Dict[str, str | list | tuple] = {}

    def filter_queryset(self, queryset)->QuerySet[T]:
        queryset: QuerySet[T]= super().filter_queryset(queryset)
        sorting: str = self.request.GET.get("sorting")
        if len(sorting or ''):
            try:
                def sort_and_return(queryset:QuerySet,sorting:str):
                    pks=list(queryset.order_by('pk').distinct('pk').values_list('pk',flat=True))
                    qs=queryset.model.objects.filter(pk__in=pks).order_by(sorting)
                    return qs.distinct()
                aliases = self.sorting_aliases
                for k, v in aliases.items():
                    neutral = sorting.replace("-", "")
                    prefix = neutral != sorting and "-" or ""

                    if isinstance(v, (list, tuple)) and k == neutral:
                        if isinstance(v[1],types.FunctionType):
                            return v[1](queryset).order_by(f'{prefix}{v[0]}')
                        return sort_and_return(queryset.alias(**{v[0]: v[1]}),f"{prefix}{v[0]}")
                    if k == neutral:
                        sorting = f"{prefix}{v}"
                    break
                sorting = sorting.replace(".", "__")
                return sort_and_return(queryset,sorting)
            except Exception as e:
                ...
        return queryset


class ViewsetPropsMixin(Generic[T]):
    @property
    def filtered_qs(self)->QuerySet[T]:
        return self.filter_queryset(self.get_queryset())
    
    
    @property
    def qs(self)->QuerySet[T]:
        return self.get_queryset()


__title__ = 'Djangondor'
__version__ = '1.8.1'
__author__ = 'Dalitso Sakala <dalitso.1sc@gmail.com>'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2024-2025'

# Version synonym
VERSION = __version__



import re
from urllib.parse import urlparse
from django.shortcuts import render
from django.db.models import QuerySet
from django.core.paginator import Paginator
from typing import Dict, Iterable
from  django.http import HttpRequest, JsonResponse

# Create your views here.

def json_response(data:list|tuple|QuerySet|Dict|str=None,status=None,meta=None,next:str=None,prev:str=None,count=None,**kwargs):
    '''Generates a JSONResponse
    * When `data` is an instance of `list` a response with
    {count, and results} is returned
    * When `data` is a str, an error response is returned
    with {error}
    * When data is `None` or no argument is passed, a blank response is returned
    '''
    is_list=isinstance(data,(list,tuple,set,QuerySet))
    if data or  (isinstance(data,dict) and kwargs) or is_list:
        is_error=isinstance(data,str)
        
        response=JsonResponse(
            {
                'count':count or len(list(data)),
                'results':list(data),
                'meta':meta,
                'next':next,
                'previous':prev,
            }
            if is_list\
            else {'error':data} if is_error else \
                {**data,**kwargs} if isinstance(data,dict) else data
        )
        response.status_code=status or 400 if is_error else status or 200
    else:
        response=JsonResponse(None,safe=True)
        response.status_code=status
    
    return response



def generate_pagination(request:HttpRequest,items:Iterable,page_number=1,page_size=40,generate_urls=True):
    '''
    Create a page from the given `items` list.
    RETURNS

        `page:QuerySet`, `next_url:str|None`, `count:int`, `prev:str|None`
    '''
    pag=Paginator(items,page_size)
    page=pag.page(page_number)
    page_by1=[]
    lst=page.object_list


    
    url=request.build_absolute_uri()
    next,prev=None,None
    if generate_urls:
        url=re.sub(r'page\=\d*','',url)
        urlp=urlparse(url)
        next= f"{urlp.geturl()}{'&' if urlp.query else '?'}page={page.next_page_number()}" if page.has_next() else None
        prev=f"{urlp.geturl()}{'&' if urlp.query else '?'}page={page.previous_page_number()}" if page.has_previous() else None
    return lst,next,pag.count, prev
